[{"locale":"es"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Alfa"}},"key":"1D6E2"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Beta"}},"key":"1D6E3"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Gamma"}},"key":"1D6E4"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Delta"}},"key":"1D6E5"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Épsilon"}},"key":"1D6E6"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Zeta"}},"key":"1D6E7"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Eta"}},"key":"1D6E8"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Theta"}},"key":"1D6E9"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Iota"}},"key":"1D6EA"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Kappa"}},"key":"1D6EB"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Lambda"}},"key":"1D6EC"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Mi"}},"key":"1D6ED"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Ni"}},"key":"1D6EE"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Xi"}},"key":"1D6EF"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Ómicron"}},"key":"1D6F0"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Pi"}},"key":"1D6F1"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Rho"}},"key":"1D6F2"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Sigma"}},"key":"1D6F4"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Tau"}},"key":"1D6F5"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Ípsilon"}},"key":"1D6F6"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Phi"}},"key":"1D6F7"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Ji"}},"key":"1D6F8"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Psi"}},"key":"1D6F9"},{"category":"Lu","mappings":{"default":{"default":"cursiva mayúscula Omega"}},"key":"1D6FA"},{"category":"Ll","mappings":{"default":{"default":"cursiva alfa"}},"key":"1D6FC"},{"category":"Ll","mappings":{"default":{"default":"cursiva beta"}},"key":"1D6FD"},{"category":"Ll","mappings":{"default":{"default":"cursiva gamma"}},"key":"1D6FE"},{"category":"Ll","mappings":{"default":{"default":"cursiva delta"}},"key":"1D6FF"},{"category":"Ll","mappings":{"default":{"default":"cursiva épsilon"}},"key":"1D700"},{"category":"Ll","mappings":{"default":{"default":"cursiva zeta"}},"key":"1D701"},{"category":"Ll","mappings":{"default":{"default":"cursiva eta"}},"key":"1D702"},{"category":"Ll","mappings":{"default":{"default":"cursiva theta"}},"key":"1D703"},{"category":"Ll","mappings":{"default":{"default":"cursiva iota"}},"key":"1D704"},{"category":"Ll","mappings":{"default":{"default":"cursiva kappa"}},"key":"1D705"},{"category":"Ll","mappings":{"default":{"default":"cursiva lambda"}},"key":"1D706"},{"category":"Ll","mappings":{"default":{"default":"cursiva mi"}},"key":"1D707"},{"category":"Ll","mappings":{"default":{"default":"cursiva ni"}},"key":"1D708"},{"category":"Ll","mappings":{"default":{"default":"cursiva xi"}},"key":"1D709"},{"category":"Ll","mappings":{"default":{"default":"cursiva ómicron"}},"key":"1D70A"},{"category":"Ll","mappings":{"default":{"default":"cursiva pi"}},"key":"1D70B"},{"category":"Ll","mappings":{"default":{"default":"cursiva rho"}},"key":"1D70C"},{"category":"Ll","mappings":{"default":{"default":"cursiva final sigma"}},"key":"1D70D"},{"category":"Ll","mappings":{"default":{"default":"cursiva sigma"}},"key":"1D70E"},{"category":"Ll","mappings":{"default":{"default":"cursiva tau"}},"key":"1D70F"},{"category":"Ll","mappings":{"default":{"default":"cursiva ípsilon"}},"key":"1D710"},{"category":"Ll","mappings":{"default":{"default":"cursiva phi"}},"key":"1D711"},{"category":"Ll","mappings":{"default":{"default":"cursiva ji"}},"key":"1D712"},{"category":"Ll","mappings":{"default":{"default":"cursiva psi"}},"key":"1D713"},{"category":"Ll","mappings":{"default":{"default":"cursiva omega"}},"key":"1D714"}]
